/*******************************************************************************
* Title                 :   Pulse click Demonstration
* Filename              :   PulseExampleFT90.c
* Author                :   RBL
* Origin Date           :   15/12/2015
* Notes                 :   None
*******************************************************************************/
/*************** MODULE REVISION LOG ******************************************
*
*    Date    Software Version    Initials   Description
*  15/12/15        .1               RBL     Module Created.
*  16/12/15        .2               RBL     Correction of Frequency
*
*******************************************************************************/
/**
 * @file PulseExampleFT90.c
 * @brief Example of Pulse click operation.
 */
/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>
#include <stdbool.h>
#include "built_in.h"
#include "resources.h"

/******************************************************************************
* Module Preprocessor Constants
*******************************************************************************/
#define IRQ_DISABLE 1
#define IRQ_ENABLE 0
#define PULSE_OUTPUT_PIN 9

/******************************************************************************
* Module Preprocessor Macros
*******************************************************************************/

/******************************************************************************
* Module Typedefs
*******************************************************************************/

/******************************************************************************
* Module Variable Definitions
*******************************************************************************/
// TFT module connections
char TFT_DataPort at GPIO_PORT_32_39;
sbit TFT_RST at GPIO_PIN42_bit;
sbit TFT_RS at GPIO_PIN40_bit;
sbit TFT_CS at GPIO_PIN41_bit;
sbit TFT_RD at GPIO_PIN46_bit;
sbit TFT_WR at GPIO_PIN47_bit;
sbit TFT_BLED at GPIO_PIN58_bit;
// End TFT module connections

volatile double pulse_count;
volatile double frequency;
volatile bool update_time;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
void init_timer_a( void );
void system_init( void );
void auxiliary_code( void );
void set_index( unsigned short index );
void write_command( unsigned short cmd );
void write_data( unsigned int _data );
void tft_initialize( void );
void tft_update( void );

/******************************************************************************
* Function Definitions
*******************************************************************************/
void init_timer_a()
{
    // 1250ms Timer
    TIMER_CONTROL_0 = 2;
    TIMER_SELECT = 0;
    TIMER_PRESC_LS = 36;
    TIMER_PRESC_MS = 244;
    TIMER_WRITE_LS = 207;
    TIMER_WRITE_MS = 7;
    TIMER_CONTROL_3 = 0;
    TIMER_CONTROL_4 |= 17;
    TIMER_CONTROL_2 |= 16;
    TIMER_INT |= 2;
    TIMER_CONTROL_1 |= 1;
}

void auxiliary_code()
{
    TFT_WR = 0;
    asm nop;
    TFT_WR = 1;
}

void set_index( unsigned short index )
{
    TFT_RS = 0;
    asm nop;
    TFT_DataPort = index;
    asm nop;
    asm nop;
    auxiliary_code();
}

void write_command( unsigned short cmd )
{
    TFT_RS = 1;
    asm nop;
    TFT_DataPort = cmd;
    asm nop;
    asm nop;
    auxiliary_code();
}

void write_data( unsigned int _data )
{
    TFT_RS = 1;
    asm nop;
    TFT_DataPort = Hi( _data );
    asm nop;
    asm nop;
    auxiliary_code();
    asm nop;
    TFT_DataPort = Lo( _data );
    asm nop;
    asm nop;
    auxiliary_code();
}

void tft_initialize()
{
    TFT_Set_Pen( CL_WHITE, 1 );
    TFT_Set_Brush( 1, CL_WHITE, 0, 0, 0, 0 );
    TFT_Set_Font( TFT_defaultFont, CL_BLACK, FO_HORIZONTAL );

    TFT_Fill_Screen( CL_WHITE );
    TFT_Set_Pen( CL_Black, 1 );
    TFT_Line( 20, 220, 300, 220 );
    TFT_Line( 20,  46, 300,  46 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Pulse click", 110, 14 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL );
    TFT_Write_Text( "EasyMx PRO v7 for STM32", 19, 223 );
    TFT_Set_Font( &Verdana12x13_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "www.mikroe.com", 200, 223 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_RED, FO_HORIZONTAL );
    TFT_Write_Text( "Frequency", 110, 100 );
}

void tft_update()
{
    char tmp_txt[20];
    static double last_frequency;

    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL );
    sprintf( tmp_txt, ( last_frequency > 1000.00 ) ? "%6.1f Hz" : "%3.1f Hz" ,
             last_frequency );
    TFT_Write_Text( tmp_txt, 120, 130 );
    TFT_Set_Font( &HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL );
    sprintf( tmp_txt, ( frequency > 1000.00 ) ? "%6.1f Hz" : "%3.1f Hz",
             frequency );
    TFT_Write_Text( tmp_txt, 120, 130 );
    last_frequency = frequency;

    update_time = false;
}

void system_init()
{
    GLOBAL_INTERRUPT_MASK_bit = IRQ_DISABLE;  // Disable all interrupts
    // Pulse
    GPIO_Config( &GPIO_PORT_08_15, _GPIO_PINMASK_1, _GPIO_DIR_INPUT,
                 _GPIO_CFG_PULL_UP | _GPIO_CFG_DRIVE_16mA |
                 _GPIO_CFG_SCHMITT_DISABLE | _GPIO_CFG_SLEW_RATE_FAST );
    GPIO_CFG_08_15 |= ( 1 << 5 ) | ( 0 << 4 );   // Enable interrupts for GPIO
    GPIO_INT_EN_00_31 |= ( 1 << PULSE_OUTPUT_PIN );
    // TFT
    TFT_Set_Active( set_index, write_command, write_data );
    TFT_Init_ILI9341_8bit( 320, 240 );
    tft_initialize();
    // Timer
    init_timer_a();

    pulse_count = 0;

    GLOBAL_INTERRUPT_MASK_bit = IRQ_ENABLE;  // Enable all interrupts
}

void main()
{
    system_init();

    while( 1 )
    {
        if( update_time )
            tft_update();
    }
}

void pulse_ISR() iv IVT_GPIO_IRQ ics ICS_AUTO
{
    if( GPIO_INT_PEND_00_31 & ( 1 << PULSE_OUTPUT_PIN ) )
    {
        pulse_count += 1.0f;
        GPIO_INT_PEND_00_31 |= ( 1 << PULSE_OUTPUT_PIN );
    }
}

void timer_interrupt() iv IVT_TIMERS_IRQ
{
    frequency = pulse_count * .750f;
    pulse_count = 0.0f;
    update_time = true;

    if( TIMER_INT_A_bit )
        TIMER_INT = ( TIMER_INT & 0xAA ) | ( 1 << 0 );
}

/*************** END OF FUNCTIONS ***************************************************************************/